"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = require("fs-extra");
const upath_1 = require("upath");
const vue_template_compiler_1 = require("vue-template-compiler");
const utils_1 = require("../utils");
const builder_utils_1 = require("@ewizardjs/builder-utils");
const CONFIG_JS_FILE = './ewizard.config.js';
const TEMPLATE_VUE_FILE = './ewizard.template.vue';
const CONFIG = 'config';
async function default_1(source) {
    const configPath = upath_1.resolve(this.context, CONFIG_JS_FILE);
    const exists = await fs_extra_1.pathExists(configPath);
    const customTemplatePath = upath_1.resolve(this.context, TEMPLATE_VUE_FILE);
    const customTemplateExists = await fs_extra_1.pathExists(customTemplatePath);
    if (exists) {
        const configOptions = await getConfigOptions(this.context);
        const basicReplacement = `
      ;const ${CONFIG} = require('${CONFIG_JS_FILE}').default;
      ${configOptions}
      component.options.ewizardConfig = ${CONFIG};
      try {
      /custom-template-require-marker/;
      } catch (e) {}
      export default component.exports;`;
        const customTemplateReplacement = basicReplacement.replace('/custom-template-require-marker/', `require('${TEMPLATE_VUE_FILE}')`);
        const replacement = customTemplateExists ? customTemplateReplacement : basicReplacement;
        return source.replace('export default component.exports', replacement);
    }
    return source;
}
exports.default = default_1;
async function getConfigOptions(dirPath) {
    const templateOptions = await getTemplateOptions(dirPath);
    return templateOptions;
}
async function getTemplateOptions(dirPath) {
    try {
        const templatePath = upath_1.resolve(dirPath, TEMPLATE_VUE_FILE);
        const vueFileContent = await utils_1.getNormalizedVueFileContent(templatePath);
        const { template } = vue_template_compiler_1.parseComponent(vueFileContent);
        const templateOption = getConfigProperty('template', template === null || template === void 0 ? void 0 : template.content);
        const templatePathOption = `${CONFIG}.templatePath='${TEMPLATE_VUE_FILE}';`;
        return `${templatePathOption}${templateOption}`;
    }
    catch (e) {
        return '';
    }
}
function getConfigProperty(prop, value) {
    if (value) {
        const newVal = builder_utils_1.replaceSpecialCharacters(value);
        return `${CONFIG}.${prop}='${newVal}';`;
    }
    return '';
}
//# sourceMappingURL=config-loader.js.map