"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DevBuilder = void 0;
const settings_1 = require("@ewizardjs/settings");
const system_settings_1 = require("@ewizardjs/system-settings");
const content_types_1 = require("@ewizardjs/content-types");
const path_1 = require("path");
const webpack_1 = __importStar(require("webpack"));
const webpack_dev_server_1 = __importDefault(require("webpack-dev-server"));
const util_1 = require("util");
const find_free_port_1 = __importDefault(require("find-free-port"));
const builder_1 = require("./builder");
class DevBuilder extends builder_1.Builder {
    async build(options = {}) {
        const compiler = await this.getCompiler(options);
        return util_1.promisify(compiler.run).call(compiler);
    }
    async serve(options = {}) {
        const port = await this.findPort();
        const server = await this.createServer(options);
        server.listen(port, () => {
            console.log(`\nServer is listening on port ${port}\n`);
        });
    }
    async createServer(options) {
        let configs = await this
            .getConfigWithComponentsListLibrary(Object.assign({ live: true }, options));
        if (options.layout) {
            const layoutConfig = await this.getLayoutConfig(options);
            configs = await this.getConfigWithComponentsListLibrary(options, layoutConfig);
        }
        const historyApiFallback = await this.getHistoryApiConfigurations();
        const serverConfig = {
            historyApiFallback,
            publicPath: '/',
            contentBase: this.options.presentationPath,
            hot: true,
            quiet: true,
            clientLogLevel: 'none',
            writeToDisk: (filePath) => !/hot-update\.(js|json)$/.test(filePath),
        };
        configs.forEach(config => { var _a; return (_a = config.plugins) === null || _a === void 0 ? void 0 : _a.push(new webpack_1.HotModuleReplacementPlugin()); });
        webpack_dev_server_1.default.addDevServerEntrypoints(configs, serverConfig);
        return new webpack_dev_server_1.default(webpack_1.default(configs), serverConfig);
    }
    async findPort() {
        const [port] = await find_free_port_1.default(3000);
        return port;
    }
    async getHistoryApiConfigurations() {
        const isEnabledHistoryApi = await this.isHistoryApiFallbackEnabled();
        const basePath = '/';
        if (!isEnabledHistoryApi) {
            return false;
        }
        return {
            rewrites: [
                {
                    from: /^\/.*/,
                    to: ({ request, parsedUrl }) => {
                        var _a;
                        const referer = (_a = request === null || request === void 0 ? void 0 : request.headers) === null || _a === void 0 ? void 0 : _a.referer;
                        const { pathname: resourcePathName } = parsedUrl;
                        if (!referer || !resourcePathName || isNotResourcePath(resourcePathName)) {
                            return basePath;
                        }
                        const { pathname: urlPathName } = new URL(referer);
                        const pathName = urlPathName.replace(/[^\/]*$/, '');
                        return resourcePathName.replace(pathName, '/');
                    },
                },
            ],
        };
    }
    async isHistoryApiFallbackEnabled() {
        const [isSite, isHistoryMode] = await Promise.all([
            this.isSiteContentType(),
            this.isHistoryRouterMode(),
        ]);
        return isSite && isHistoryMode;
    }
    async isSiteContentType() {
        const { content } = await system_settings_1.get(this.options.presentationPath);
        const { type, subtype } = content;
        return type === content_types_1.ContentTypes.SITE || subtype === content_types_1.ContentTypes.SITE;
    }
    async isHistoryRouterMode() {
        var _a;
        const { navigation } = await settings_1.getRaw(this.options.presentationPath);
        return ((_a = navigation === null || navigation === void 0 ? void 0 : navigation.router) === null || _a === void 0 ? void 0 : _a.mode) === 'history';
    }
}
exports.DevBuilder = DevBuilder;
function isNotResourcePath(resourcePath) {
    return !path_1.extname(resourcePath);
}
//# sourceMappingURL=devBuilder.js.map